#ifndef TIME_MANAGER_H
#define TIME_MANAGER_H

#include <chrono>
#include <thread>

#include "Minimax.h"

namespace chessy
{
    class ChessGame;
}


class UciParser;


class TimeManager
{
protected:
    using Millisecs = std::chrono::milliseconds;
    chessy::ChessGame& game;
    //chessy::MiniMax minimax;

    bool running;
    std::thread searchThread;

    UciParser& uci;

public:
    inline TimeManager(chessy::ChessGame& game, UciParser& uci) :
        game{ game }, uci{ uci } {}
    
    void startInfiniteSearch(void);

    virtual void stop(void);
    virtual void join(void);
    virtual void startSearch(void);
    virtual bool isRunning(void) const;
};


class FixedSearchTimer :
    public TimeManager
{
protected:
    Millisecs thinkTime;
    std::chrono::time_point<std::chrono::steady_clock> startPoint;
public:
    inline FixedSearchTimer(chessy::ChessGame& game, UciParser& uci) :
        TimeManager{ game, uci } {}

    inline Millisecs    getThinkTime(void) const    { return thinkTime; }
    inline void         setThinkTime(Millisecs t)   { thinkTime = t; } 

    virtual void startSearch(void) override;
    virtual void search(void);
};


#endif // TIME_MANAGER_H
